/**
 * @file Coordinate.hpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */
#pragma once

#include <cstdint>
#include <functional>

/**
 * Represents a coordinate
 */
struct Coordinate
{
	/** Returns true if the given coordinate is equal to this coordinate. */
	bool operator==(const Coordinate&) const = default;

	/**
	 * Adds the given coordinate to this coordinate.
	 */
	Coordinate& operator+=(const Coordinate& other)
	{
		x += other.x;
		y += other.y;
		return *this;
	}

	/** Contains the horizontal part of the Coordinate. Starts at the left. */
	int8_t x{ 0 };
	/** Contains the vertical part of the Coordinate. Starts at the top. */
	int8_t y{ 0 };
};

/**
 * Allows a coordinate to be stored in an unordered_map.
 */
template <>
struct std::hash<Coordinate>
{
	/**
	 * Returns a hash corresponding to the given Coordinate.
	 */
	std::size_t operator()(const Coordinate& pos) const noexcept
	{
		return pos.x << 3 | pos.y;
	}
};

/**
 * Adds two coordinates and returns a new coordinate containing the sum.
 */
inline Coordinate operator+(const Coordinate& a, const Coordinate& b)
{
	Coordinate result = a;
	result += b;
	return result;
}