/**
 * @file InitialPositionsChoice.hpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */
#pragma once

#include "SelectionWindow.hpp"
#include "Board.hpp"

namespace frontend {
    /**
     * This widget contains a screenshot of an initial configuration and a legend.
     */
    class InitialPositionsChoice :
        public QWidget
    {
    public:
        /**
         * Constructs an InitialPositionsChoice.
         *
         * @param parent The dialog containing this widget.
         * @param positions The corresponding initial configuration.
         */
        InitialPositionsChoice(SelectionWindow* parent, backend::InitialPositions positions);

    private:
        /**
         * This method reacts to a click event.
         *
         * When this widget is clicked, the main window loads the corresponding
         * initial configuration and the SelectionWindow is hidden.
         */
        void mousePressEvent(QMouseEvent* event) override;

        /**
         * The intial configuration corresponding to this widget.
         */
        backend::InitialPositions positions_;

        /**
         * The main window (this widget's grandparent).
         */
        Window* window_;

        /**
         * Associates a pretty name to each initial configuration.
         */
        static constexpr std::string_view positionsNames[] = {
            "Default",
            "Pawn Hell",
            "Mirror",
            "Clockwise"
        };
    };
}