/**
 * @file King.hpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */
#pragma once

#include "Piece.hpp"

/**
 * Represents the side a king wants to castle to.
 */
enum class CastleSide
{
	Left = 0,
	Right = 1
};

namespace backend {
	class Rook;

	/**
	 * Represents the King Piece.
	 */
	class King :
		public Piece
	{
	public:
		/** Constructs a King. */
		King(Side side, Coordinate pos);

		/** Destroys a King. */
		~King() override;

		/**
		 * Puts all the possible moves of the King in the given container.
		 */
		void computePossibleMoves(std::vector<Coordinate>&) const override;

		/**
		 * Returns the utf-8 symbol of the King depending on the color.
		 */
		std::string getSymbol() const override;

		/**
		 * If destination is a castling move, returns the corresponding
		 * rook.
		 */
		Rook* getCastleRook(Coordinate destination) const;

	protected:
		/**
		 * Adds a textual representation of the bishop to the given stream.
		 */
		std::ostream& print(std::ostream& o) const override;

	private:
		/**
		 * Returns true if the king can castle on the given side.
		 */
		bool canCastle(CastleSide side) const;

		/**
		 * Contains the number of King instances in existence.
		 */
		static unsigned int instancesCount;
	};
}
