/**
 * @file Knight.cpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */

#include "Knight.hpp"

namespace backend {
	void Knight::computePossibleMoves(std::vector<Coordinate>& possibleMoves) const
	{
		for (int8_t i = 0; i < 8; i++)
		{
			Coordinate possiblePos;

			possiblePos.x += 2 >> (i / 4);
			if (!(i / 2 % 2)) possiblePos.x = -possiblePos.x;

			possiblePos.y += 1 << (i / 4);
			if (i % 2) possiblePos.y = -possiblePos.y;

			possiblePos += getPos();

			if (!BackendArray::isOutOfBounds(possiblePos) && getBoard().getTileSide(possiblePos) != static_cast<TileType>(getSide()))
			{
				possibleMoves.push_back(possiblePos);
			}
		}
	}

	std::string Knight::getSymbol() const
	{
		if (getSide() == Side::Black) return "♞";
		else return "♘";
	}


	std::ostream& Knight::print(std::ostream& o) const
	{
		if (getSide() == Side::Black) return o << "\033[1;30mN\033[0m";
		else return o << "\033[1;37mN\033[0m";
	}
}