/**
 * @file Pawn.cpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */

#include "Pawn.hpp"

namespace backend {
	void Pawn::computePossibleMoves(std::vector<Coordinate>& possibleMoves) const
	{
		computeAttackMoves(possibleMoves);

		Coordinate possiblePos = getPos() + Coordinate{ 0, getDirection() };
		if (getBoard().getTileSide(possiblePos) == TileType::None)
		{
			possibleMoves.push_back(possiblePos);
			if (getPos().y == getStartRow())
			{
				possiblePos += Coordinate{ 0, getDirection() };
				if (getBoard().getTileSide(possiblePos) == TileType::None)
					possibleMoves.push_back(possiblePos);
			}
		}
	}

	void Pawn::computeAttackMoves(std::vector<Coordinate>& possibleMoves) const
	{
		Coordinate possiblePos = getPos() + Coordinate{ -1, getDirection() };
		if (!BackendArray::isOutOfBounds(possiblePos) && getBoard().getTileSide(possiblePos) == static_cast<TileType>(getBoard().getOppositeSide(getSide())))
			possibleMoves.push_back(possiblePos);

		possiblePos = getPos() + Coordinate{ 1, getDirection() };
		if (!BackendArray::isOutOfBounds(possiblePos) && getBoard().getTileSide(possiblePos) == static_cast<TileType>(getBoard().getOppositeSide(getSide())))
			possibleMoves.push_back(possiblePos);
	}

	bool Pawn::canAttack(Coordinate pos) const
	{
		if (!isAlive()) return false;
		std::vector<Coordinate> possibleMoves;
		computeAttackMoves(possibleMoves);
		return std::find(possibleMoves.begin(), possibleMoves.end(), pos) != possibleMoves.end();
	}

	std::string Pawn::getSymbol() const
	{
		if (getSide() == Side::Black) return "♟";
		else return "♙";
	}

	int8_t Pawn::getDirection() const
	{
		return getSide() == Side::White ? -1 : 1;
	}

	int8_t Pawn::getStartRow() const
	{
		return getSide() == Side::White ? 6 : 1;
	}

	int8_t Pawn::getFinalRow() const
	{
		return getSide() == Side::White ? 0 : 7;
	}

	std::ostream& Pawn::print(std::ostream& o) const
	{
		if (getSide() == Side::Black) return o << "\033[1;30mp\033[0m";
		else return o << "\033[1;37mp\033[0m";
	}
}