/**
 * @file Pawn.hpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */
#pragma once

#include "Piece.hpp"

namespace backend {
	/**
	 * Represents a Pawn Piece.
	 */
	class Pawn : public Piece
	{
	public:
		using Piece::Piece;

		/**
		 * Puts all the possible moves of the Pawn in the given container.
		 */
		void computePossibleMoves(std::vector<Coordinate>&) const override;

		/**
		 * Puts all the possible attacking moves of the Pawn in the given container.
		 */
		void computeAttackMoves(std::vector<Coordinate>&) const;

		/**
		 * Returns true if this pawn can attack at the given Coordinate.
		 */
		bool canAttack(Coordinate pos) const override;

		/**
		 * Returns the utf-8 symbol of the Pawn depending on the color.
		 */
		std::string getSymbol() const override;

		/**
		 * Returns the direction of this Pawn, depending on its color.
		 */
		int8_t getDirection() const;

		/**
		 * Returns the starting row of this Pawn, depending on its color.
		 */
		int8_t getStartRow() const;

		/**
		 * Returns the final row if this Pawn, depending on its color.
		 */
		int8_t getFinalRow() const;

	protected:
		/**
		 * Adds a textual representation of the Pawn to the given stream.
		 */
		std::ostream& print(std::ostream& o) const override;
	};
}