/**
 * @file Piece.cpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */

#include "Piece.hpp"

#include "Array.hpp"

namespace backend {
	Piece::Piece(Side side, Coordinate pos) : pos_(pos), side_(side)
	{
	}

	bool Piece::canAttack(Coordinate pos) const
	{
		return canMoveTo(pos);
	}

	void Piece::exploreDirection(Coordinate from, Coordinate delta, std::vector<Coordinate>& possibleMoves) const
	{
		for (Coordinate possiblePos = from; !BackendArray::isOutOfBounds(possiblePos); possiblePos += delta)
		{
			TileType type = getBoard().getTileSide(possiblePos);

			if (type == static_cast<TileType>(getSide())) break;
			else if (type == TileType::None)
			{
				possibleMoves.push_back(possiblePos);
				continue;
			}
			else
			{
				possibleMoves.push_back(possiblePos);
				break;
			}
		}
	}

	bool Piece::canMoveTo(Coordinate pos) const
	{
		if (!alive_) return false;
		std::vector<Coordinate> possibleMoves;
		computePossibleMoves(possibleMoves);
		return std::find(possibleMoves.begin(), possibleMoves.end(), pos) != possibleMoves.end();
	}

	Side Piece::getSide() const
	{
		return side_;
	}

	Board& Piece::getBoard()
	{
		return *Board::getInstance();
	}

	const Board& Piece::getBoard() const
	{
		return *Board::getInstance();
	}

	Coordinate Piece::getPos() const
	{
		return pos_;
	}

	void Piece::setPos(Coordinate pos)
	{
		pos_ = pos;
	}

	bool Piece::isAlive() const
	{
		return alive_;
	}

	void Piece::setAlive(bool alive)
	{
		alive_ = alive;
	}

	unsigned int Piece::getMoveCount() const
	{
		return moveCount_;
	}

	void Piece::incrementMoveCount()
	{
		moveCount_++;
	}

	void Piece::decrementMoveCount()
	{
		moveCount_--;
	}

	std::ostream& operator<<(std::ostream& o, const backend::Piece& piece)
	{
		return piece.print(o);
	}
}
