/**
 * @file Queen.hpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */
#pragma once

#include "Bishop.hpp"
#include "Rook.hpp"

namespace backend {
	/**
	 * Represents the Queen Piece.
	 *
	 * Inherits from Bishop and Rook so it can combine their possible moves.
	 */
	class Queen : public Bishop, public Rook
	{
	public:
		/** Constructs a Queen. */
		Queen(Side side, Coordinate pos);

		/**
		 * Puts all the possible moves of the Queen in the given container.
		 */
		void computePossibleMoves(std::vector<Coordinate>&) const override;

		/**
		 * Returns the utf-8 symbol of the Queen depending on the color.
		 */
		std::string getSymbol() const override;

	protected:
		/**
		 * Adds a textual representation of the Queen to the given stream.
		 */
		std::ostream& print(std::ostream& o) const override;
	};
}