/**
 * @file TemporaryMove.hpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */
#pragma once

#include "Move.hpp"

namespace backend {
	/**
	 * Represents a temporary Move, following the RAII architecture.
	 */
	class TemporaryMove : public Move
	{
	public:
		/**
		 * Constructs a TemporaryMove.
		 *
		 * Sends all the arguments to the Move constructor, then executed the
		 * Move.
		 */
		TemporaryMove(Coordinate from, Coordinate to, bool verify = true);

		/**
		 * Destructs a TemporaryMove.
		 *
		 * Cancels the Move before destroying it.
		 */
		~TemporaryMove();
	};
}