/**
 * @file Tile.cpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */

#include "Tile.hpp"

#include <QMouseEvent>

namespace frontend {
	Tile::Tile(Window& window, Coordinate pos) : window_(window), pos_(pos)
	{
		setFont(QFont("Times", height() / 10));
		setAlignment(Qt::AlignCenter);

		resetColor();
	}

	void Tile::setColor(const std::string& color)
	{
		setStyleSheet(QString::fromStdString("background-color: " + color));
	}

	Coordinate Tile::getPos() const
	{
		return pos_;
	}

	void Tile::setPieceText(backend::Piece* piece)
	{
		setText(piece == nullptr ? "" : QString::fromStdString(piece->getSymbol()));
	}

	void Tile::highlight()
	{
		// Adds a round border around the tile.
		setStyleSheet(styleSheet() + ";border:5px solid red;border-radius: 10px;margin: 1px;");
	}

	void Tile::resetColor()
	{
		if ((pos_.x + pos_.y) % 2 == 0)
			setColor("hsl(5, 30%, 93%)");
		else
			setColor("hsl(3, 35%, 47%)");
	}
}