/**
 * @file Tile.hpp
 * @author Romain Lebbadi-Breteau
 * @version 2.1
 * @date 2022-05-06
 *
 * @copyright Copyright (c) 2022
 *
 * By the way, c'est quoi le STEP ?
 *
 */
#pragma once

#include <QLabel>

#include "Window.hpp"

namespace frontend {
	class Window;

	/**
	 * Represents a Tile of the Board in the view.
	 */
	class Tile : public QLabel
	{
	public:
		/** Constructs a Tile */
		Tile(Window& window, Coordinate pos);

		/**
		 * Sets the color of the tile using CSS format;
		 */
		void setColor(const std::string& color);

		/**
		 * Getter for the Tile position.
		 */
		Coordinate getPos() const;

		/**
		 * Sets the text of the Tile using the Piece it contains.
		 *
		 * @param piece the Piece this Tile contains or nullptr;
		 */
		void setPieceText(backend::Piece* piece);

		/**
		 * Highlights this Tile to indicate a possible Move.
		 */
		void highlight();

	public slots:
		/**
		 * Resets the color of the Tile to the default one.
		 */
		void resetColor();

	private:
		/**
		 * Contains a reference to the main Window.
		 */
		Window& window_;

		/**
		 * Contains the Tile's position on the Board.
		 */
		Coordinate pos_;
	};
}